package org.rcisoft.business.barrange.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.bslschedule.service.BSlScheduleService;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.core.converter.MultipartFile2HSSFWorkbookConverter;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.barrange.entity.BArrange;
import org.rcisoft.business.barrange.service.BArrangeService;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

/**
* Created by  on 2018-4-19 10:28:59.
*/
@RestController
@RequestMapping("/barrange")
public class BArrangeController extends PaginationController<BArrange> {

    @Autowired
    private BArrangeService bArrangeServiceImpl;

    @Autowired
    private BSlScheduleService bSlScheduleServiceImpl;

    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(String json) {
        return Result.builder(bArrangeServiceImpl.save(json),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                json);
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @ApiImplicitParam(name = "termCode", value = "学期编号", required = false, dataType = "varchar")
    @GetMapping(value = "/findAllByPagination")
    public GridModel findAllByTermCodeByPagination(BArrange bArrange) {
        bArrangeServiceImpl.findAllByPagination(getPaginationUtility(),bArrange);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="企业查看个人排课信息", notes="企业查看个人排课信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "agencyCode", value = "企业编号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "status", value = "状态", required = false, dataType = "varchar")})
    @GetMapping(value = "/findAllByAgencyCodeByPagination")
    public GridModel findAllByAgencyCodeByPagination(@ApiIgnore BArrange bArrange) {
        bArrangeServiceImpl.findAllByAgencyCodeByPagination(getPaginationUtility(),bArrange);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="excel导入", notes="excel导入排课信息")
    @ApiImplicitParam(name = "importFile", value = "excel文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1001')")
    @PostMapping(value = "excelImport")
    public Result excelImport(MultipartFile importFile,String subAgencyId,String termCode) throws Exception {
        PersistModel data = bArrangeServiceImpl.importExcel(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken(),subAgencyId);
        bSlScheduleServiceImpl.importFile(MultipartFile2HSSFWorkbookConverter.convert(importFile),getToken(),subAgencyId,termCode);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                data.getInfluenceReason());
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/findDtoBybSubtaskId")
    public Result findDtoBybSubtaskId(String bSubtaskId) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bArrangeServiceImpl.queryDtoBySubtaskId(bSubtaskId));
    }


    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/findDtoByMany")
    public Result findDtoByMany(String agencyCode,String name,String termCode,String teacherCode) {
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bArrangeServiceImpl.queryDtoByMany(agencyCode, name, termCode,teacherCode));
    }

    @ApiOperation(value="排课审核通过后的开课接口", notes="排课审核通过后的开课接口")
    @GetMapping(value = "/changeArrangeToSl")
    public Result changeArrangeToSl(String businessId,String subTaskId,String termCode) {
        int data = bArrangeServiceImpl.changeArrangeToSl(businessId,subTaskId,termCode);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }


}
