package org.rcisoft.business.barrange.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.gson.JsonObject;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.barrange.dao.BArrangeRepository;
import org.rcisoft.business.barrange.entity.BArrange;
import org.rcisoft.business.barrange.entity.BSubAgency;
import org.rcisoft.business.barrange.entity.BSubtask;
import org.rcisoft.business.barrange.entity.ScheduleDto;
import org.rcisoft.business.barrange.service.BArrangeService;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bclass.dao.BClassRepository;
import org.rcisoft.business.bdirection.dao.BDirectionRepository;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.entity.BLessonDirection;
import org.rcisoft.business.brooms.dao.BRoomsRepository;
import org.rcisoft.business.bsl.dao.BSlRepository;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by  on 2018-4-19 10:28:59.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BArrangeServiceImpl implements BArrangeService {

    @Autowired
    private BArrangeRepository bArrangeRepository;

    @Autowired
    private BTermRepository bTermRepository;

    @Autowired
    private BSlRepository bSlRepository;

    @Autowired
    private BChapterService chapterService;

    @Autowired
    private BLessonRepository bLessonRepository;

    @Autowired
    private BDirectionRepository bDirectionRepository;

    @Autowired
    private BClassRepository bClassRepository;

    @Autowired
    private BRoomsRepository bRoomsRepository;





    /**
    * 保存 bArrange
    * @param json
    * @return
    */
//    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(String json){
        BArrange bArrange = new BArrange();
        List<BSubtask> subtaskList = new ArrayList<>();
        JSONObject jsonObject = JSONObject.fromObject(json);
        JSONObject arrange = jsonObject.getJSONObject("arrange");
        bArrange.setDescription(arrange.getString("desc"));
        bArrange.setTermCode(arrange.getString("termCode"));
        bArrange.setStatus("0");
        UserUtil.setCurrentPersistOperation(bArrange);
        int line = bArrangeRepository.insertSelective(bArrange);
        JSONArray childList = arrange.getJSONArray("childList");
        for(int i = 0; i < childList.size() ; i++){
            JSONObject subtask = childList.getJSONObject(i);
            BSubtask bSubtask = new BSubtask();
            List<BSubAgency> bSubAgencyList = new ArrayList<>();
            bSubtask.setName(subtask.getString("name"));
            bSubtask.setArrangeId(bArrange.getBusinessId());
            bSubtask.setStatus("0");
            UserUtil.setCurrentPersistOperation(bSubtask);
            subtaskList.add(bSubtask);
            JSONArray childList2 = subtask.getJSONArray("childList");

            for(int j = 0; j < childList2.size() ; j++){
                JSONObject agency = childList2.getJSONObject(j);
                BSubAgency bSubAgency = new BSubAgency();
                bSubAgency.setAgencyCode(agency.getString("agencyCode"));
                bSubAgency.setStatus("0");
                bSubAgency.setSubtaskId(bSubtask.getBusinessId());
                UserUtil.setCurrentPersistOperation(bSubAgency);
                bSubAgencyList.add(bSubAgency);
            }
            bArrangeRepository.insertBSubAgency(bSubAgencyList);
        }
        bArrangeRepository.insertBSubtask(subtaskList);
        return new PersistModel(line);
    }

    /**
    * 分页查询 bArrange
    * @return
    */
    @Override
    public List<BArrange> findAllByPagination(PageUtil paginationUtility, BArrange bArrange){
        List<BArrange> list =   bArrangeRepository.queryAll(bArrange);
        return list;
    }

    @Override
    public List<BArrange> findAllByAgencyCodeByPagination(PageUtil paginationUtility,BArrange bArrange) {
        return bArrangeRepository.queryByAgencyCode(bArrange);
    }

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public PersistModel importExcel(HSSFWorkbook hwb, String token , String subAgencyId) {
        ArrayList<ScheduleDto> scheduleDtos = new ArrayList<ScheduleDto>();

        String[] headers = {"课程编号", "教师编号", "班级编号","教室编号","周次","周学时","周一",
                "周二","周三","周四","周五","周六"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true);  //获取excel数据

        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }

        List<String> lessonCodes = new ArrayList<>();
        List<String> teacherCodes = new ArrayList<>();
        List<String> classCodes = new ArrayList<>();
        List<String> roomCodes = new ArrayList<>();

        String[] wholeValue = new String[11];
        for (String[] value : values) {
            ScheduleDto scheduleDto = new ScheduleDto();

            //每一次遇到完整数据时将完整数据存放起来
            if(!value[0].equals("")&&!value[1].equals("")&&!value[2].equals("")&&!value[3].equals("")){
                wholeValue = value;
            }

            //判断1:根据课程编号查询是否有此门课程；
            if(value[0].equals("")){
                value[0] = wholeValue[0];
            }
//            if(bLessonRepository.selectOne(new BLesson(value[0]))==null){
//                lessonCodes.add(value[0]);
//                continue;
//            }

            //判断2:根据教师编号查询是否有此教师；
            if(value[1].equals("")){
                value[1] = wholeValue[1];
            }
            SysUser user1 =new SysUser();
            user1.setLoginName(value[1]);
            user1.setDelFlag("0");
            user1.setFlag("1");
//            if (sysUserMapper.selectOne(user1) == null) {
//                teacherCodes.add(value[1]);
//                continue;
//            }

            //判断3:根据班级编号查询是否有此班级；
            if(value[2].equals("")){
                value[2] = wholeValue[2];
            }
//            for(String classCode : value[2].split(",")){
//                if(bClassRepository.selectOne(new BClass(classCode))==null){
//                    classCodes.add(classCode);
//                    continue;
//                }
//            }

            //判断4:根据教室编号查询是否有此教室；
            if(value[3].equals("")){
                value[3] = wholeValue[3];
            }
//            if(bRoomsRepository.selectOne(new BRooms(value[3]))==null){
//                roomCodes.add(value[3]);
//                continue;
//            }

            //合法:通过了四次判断，证明可建立该排课
            scheduleDto.setLessonCode(value[0]);
            scheduleDto.setTeacherCode(value[1]);
            scheduleDto.setClassCode(value[2]);
            scheduleDto.setRoomCode(value[3]);
            scheduleDto.setWeek(value[4]);
            scheduleDto.setClassHour(value[5]);
            scheduleDto.setMonday(value[6]);
            scheduleDto.setTuesday(value[7]);
            scheduleDto.setWednesday(value[8]);
            scheduleDto.setThursday(value[9]);
            scheduleDto.setFriday(value[10]);
            scheduleDto.setSaturday(value[11]);
            scheduleDto.setSubAgencyId(subAgencyId);
            scheduleDto.setCommonBusinessId();
            UserUtil.setCurrentPersistOperation(scheduleDto);
            scheduleDtos.add(scheduleDto);

        }


        String result = "";
        if (scheduleDtos.size()>0){
            int line1 = bArrangeRepository.insertList(scheduleDtos);
            if (line1 < 1 ) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                result+="成功导入"+scheduleDtos.size()+"条数据。";
            }
        }

        if(lessonCodes.size()>0){
            result+="以下课程编号不存在："+ JSON.toJSONString(lessonCodes)+"。";
        }
        if(teacherCodes.size()>0){
            result+="以下教师编号不存在："+ JSON.toJSONString(teacherCodes)+"。";
        }
        if(classCodes.size()>0){
            result+="以下班级编号不存在："+ JSON.toJSONString(classCodes)+"。";
        }
        if(roomCodes.size()>0){
            result+="以下教室编号不存在："+ JSON.toJSONString(roomCodes)+"。";
        }

        return new PersistModel(1, result);
    }

    @Override
    public List<ScheduleDto> queryDtoBySubtaskId(String bSubtaskId) {
        return bArrangeRepository.queryDtoByAgencySubtaskId(bSubtaskId);
    }

    @Override
    public List<ScheduleDto> queryDtoByMany(String agencyCode,String name,String termCode,String teacherCode) {
        return bArrangeRepository.queryDtoByAgencyMany(agencyCode,name,termCode,teacherCode);
    }

    @Override
    public int changeArrangeToSl(String businessId, String subTaskId ,String termCode) {
        int result = 0;
        List<String> subTaskStatus = bArrangeRepository.querySubTaskStatus(subTaskId);
        if(subTaskStatus.size()>1||subTaskStatus.get(0).equals("0")||subTaskStatus.get(0).equals("1")){
            //当前子任务下有教学单位未完成审核，不能进行开课
            throw new ServiceException(ResultServiceEnums.AGENCY_NOT_PASS);
        }
        List<String> subAgencyIds = bArrangeRepository.querySubAgencyId(subTaskId);
        if(subAgencyIds.size()<1){
            //子任务下没有教学单位
            throw new ServiceException(ResultServiceEnums.NO_AGENCY_SUBMISSION);
        }
        List<BSl> sls = new ArrayList<>();
        for(String subAgencyId : subAgencyIds){
            //查询要开的课
            List<ScheduleDto> schedules = bArrangeRepository.queryDtoBySubAgencyId(subAgencyId);
            for(ScheduleDto schedule : schedules){
                BSl sl = new BSl();
                BLesson bLesson = bLessonRepository.queryBLessonByCode(schedule.getLessonCode());

                //学期编号+学期中存放的开课序号最大值+1作为新的开课序号
                int maxSlCode = bTermRepository.queryMaxSlCodeByTermCode(termCode);
                //验证开课序号是否重复
                String slCode = "";
                for(int i=1;i<99999;i++){
                    slCode = termCode+String.valueOf(maxSlCode+i);
                    if(bSlRepository.selectOne(new BSl(slCode,"0","1"))==null)
                        break;
                }
                sl.setCode(slCode);//课序号
                sl.setLessonCode(schedule.getLessonCode());//课程编号
                sl.setTeacherCode(schedule.getTeacherCode());//教师编号
                sl.setTermCode(termCode);//学期编号
                sl.setAuditStatus("0");//审核状态
                sl.setClassHour(bLesson.getClassHour());//课时
                sl.setCredits(bLesson.getCredits());//学分
                sl.setSource("1");
                UserUtil.setCurrentPersistOperation(sl);
                sls.add(sl);
                bTermRepository.MaxSlCodeByTermCode(termCode);//将该学期的开课序号最大值+1
                //课程方向
                BLessonDirection bLessonDirection = new BLessonDirection();
                bLessonDirection.setBusinessId(IdGen.uuid());
                bLessonDirection.setSlId(sl.getBusinessId());
                bLessonDirection.setDirectionId(bLesson.getDirectionId());
                bDirectionRepository.insertBLessonDirection(bLessonDirection);

                //复制章节及文件目录
                chapterService.addBslFormLesson(null,sl.getLessonCode(), sl.getBusinessId());
            }
            //批量插入
            result =bSlRepository.insertList(sls);
        }
        return result;
    }

}
