package org.rcisoft.business.brooms.service.impl;

import org.rcisoft.business.brooms.utils.Upload2DataBase;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.brooms.dao.BRoomsRepository;
import org.rcisoft.business.brooms.entity.BRooms;
import org.rcisoft.business.brooms.service.BRoomsService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-4-23 9:26:21.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BRoomsServiceImpl implements BRoomsService {

    @Autowired
    private BRoomsRepository bRoomsRepository;


    /**
    * 保存 bRooms
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BRooms bRooms){
        bRooms.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(bRooms);
        int line = bRoomsRepository.insertSelective(bRooms);
        log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            bRooms.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BRooms bRooms){
        UserUtil.setCurrentMergeOperation(bRooms);
        bRooms.setDeleted();
        int line = bRoomsRepository.logicalDelete(bRooms);
        log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        bRooms.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bRooms
    * @param bRooms
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BRooms bRooms){
        UserUtil.setCurrentMergeOperation(bRooms);
        int line = bRoomsRepository.updateByPrimaryKeySelective(bRooms);
        log.info(UserUtil.getUserInfoProp(bRooms.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bRooms.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bRooms
    * @param id
    * @return
    */
    public BRooms findById(String id){
        return bRoomsRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bRooms
    * @param bRooms
    * @return
    */
    public List<BRooms> findAllByPagination(PageUtil<BRooms> paginationUtility,
        BRooms bRooms){
        bRooms.setStart();
        bRooms.setNotDeleted();
        return bRoomsRepository.queryBRoomss(bRooms);
    }

    @Override
    public Result uploadExcel(String path) {
        Map<String,Object> map = Upload2DataBase.DoIt(path,bRoomsRepository.queryAllCode());
        List<BRooms> list = (List<BRooms>) map.get("list");
        if(list.size() > 0){
            bRoomsRepository.addExcel(list);
        }
        return Result.builder(new PersistModel(list.size()),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                "成功插入 " + ((List<BRooms>) map.get("list")).size() + " 条数据，" +map.get("errorDetail"));
    }

}
