package org.rcisoft.business.bslschedule.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.rcisoft.business.barrange.entity.ScheduleDto;
import org.rcisoft.business.bcoursecode.dao.BCourseCodeRepository;
import org.rcisoft.business.bcoursecode.entity.BCourseCode;
import org.rcisoft.business.bsl.entity.BSl;
import org.rcisoft.business.bslschedule.entity.ScheduleResult;
import org.rcisoft.business.bslschedule.util.FindTime;
import org.rcisoft.business.bterm.dao.BTermRepository;
import org.rcisoft.business.bterm.entity.BTerm;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.ExcelUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.bslschedule.dao.BSlScheduleRepository;
import org.rcisoft.business.bslschedule.entity.BSlSchedule;
import org.rcisoft.business.bslschedule.service.BSlScheduleService;


import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-5-2 16:39:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BSlScheduleServiceImpl implements BSlScheduleService {

    @Autowired
    private BSlScheduleRepository bSlScheduleRepository;

    @Autowired
    private BTermRepository bTermRepository;

    @Autowired
    private BCourseCodeRepository bCourseCodeRepository;

    private static Map<String,Date> timeMap;

    private static String termCode;

    /**
    * 保存 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BSlSchedule bSlSchedule){
        bSlSchedule.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(bSlSchedule);
        int line = bSlScheduleRepository.insertSelective(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(BSlSchedule bSlSchedule){
        UserUtil.setCurrentMergeOperation(bSlSchedule);
        bSlSchedule.setDeleted();
        int line = bSlScheduleRepository.logicalDelete(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BSlSchedule bSlSchedule){
        UserUtil.setCurrentMergeOperation(bSlSchedule);
        int line = bSlScheduleRepository.updateByPrimaryKeySelective(bSlSchedule);
        log.info(UserUtil.getUserInfoProp(bSlSchedule.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            bSlSchedule.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 bSlSchedule
    * @param id
    * @return
    */
    public BSlSchedule findById(String id){
        return bSlScheduleRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 bSlSchedule
    * @param bSlSchedule
    * @return
    */
    public List<BSlSchedule> findAllByPagination(PageUtil<BSlSchedule> paginationUtility,
        BSlSchedule bSlSchedule){
        bSlSchedule.setStart();
        bSlSchedule.setNotDeleted();
        return bSlScheduleRepository.queryBSlSchedules(bSlSchedule);
    }

    @Override
    public PersistModel importFile(HSSFWorkbook hwb, String token, String subAgencyId, String termCode1) {
        termCode = termCode1;
        BTerm bTerm = bTermRepository.queryBtermByCode(termCode).get(0);
        //获取时间集合
        timeMap = FindTime.findTime(bTerm.getStartDate(),bTerm.getEndDate());
        String[] headers = {"课程编号", "教师编号", "班级编号","教室编号","周次","周学时","周一",
                "周二","周三","周四","周五","周六"};
        ArrayList<String[]> values = ExcelUtil.importExcel(hwb, headers,true);  //获取excel数据
        List<BSlSchedule> list = new ArrayList<>();
        if (values.size()<=0){
            throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DATA_NOT_EXIST);
        }
        List<String> lessonCodes = new ArrayList<>();
        List<String> teacherCodes = new ArrayList<>();
        List<String> classCodes = new ArrayList<>();
        List<String> roomCodes = new ArrayList<>();

        String[] wholeValue = new String[11];
        for (String[] value : values) {

            //每一次遇到完整数据时将完整数据存放起来
            if(!value[0].equals("")&&!value[1].equals("")&&!value[2].equals("")&&!value[3].equals("")){
                wholeValue = value;
            }

            //判断1:根据课程编号查询是否有此门课程；
            if(value[0].equals("")){
                value[0] = wholeValue[0];
            }
//            if(bLessonRepository.selectOne(new BLesson(value[0]))==null){
//                lessonCodes.add(value[0]);
//                continue;
//            }

            //判断2:根据教师编号查询是否有此教师；
            if(value[1].equals("")){
                value[1] = wholeValue[1];
            }
            SysUser user1 =new SysUser();
            user1.setLoginName(value[1]);
            user1.setDelFlag("0");
            user1.setFlag("1");
//            if (sysUserMapper.selectOne(user1) == null) {
//                teacherCodes.add(value[1]);
//                continue;
//            }

            //判断3:根据班级编号查询是否有此班级；
            if(value[2].equals("")){
                value[2] = wholeValue[2];
            }
//            for(String classCode : value[2].split(",")){
//                if(bClassRepository.selectOne(new BClass(classCode))==null){
//                    classCodes.add(classCode);
//                    continue;
//                }
//            }

            //判断4:根据教室编号查询是否有此教室；
            if(value[3].equals("")){
                value[3] = wholeValue[3];
            }
//            if(bRoomsRepository.selectOne(new BRooms(value[3]))==null){
//                roomCodes.add(value[3]);
//                continue;
//            }

            String week = value[4];
            int location = week.indexOf("-");
            String start = week.substring(0,location);
            int numStart = Integer.valueOf(start);
            String end = week.substring(location + 1,week.length());
            int numEnd = Integer.valueOf(end);

            //按周分割
            for(int i = numStart ; i <= numEnd ; i++){
                //如果周一的课不唯一.......以此类推
                if(value[6].contains("-")){
                    //找到这节课所对应的时间
                    Date date = timeMap.get(i + ",1");
                    list.addAll(setDetail(6,value,subAgencyId,date));
                }
                if(value[7].contains("-")){
                    Date date = timeMap.get(i + ",2");
                    list.addAll(setDetail(7,value,subAgencyId,date));
                }
                if(value[8].contains("-")){
                    Date date = timeMap.get(i + ",3");
                    list.addAll(setDetail(8,value,subAgencyId,date));
                }
                if(value[9].contains("-")){
                    Date date = timeMap.get(i + ",4");
                    list.addAll(setDetail(9,value,subAgencyId,date));
                }
                if(value[10].contains("-")){
                    Date date = timeMap.get(i + ",5");
                    list.addAll(setDetail(10,value,subAgencyId,date));
                }
                if(value[11].contains("-")){
                    Date date = timeMap.get(i + ",6");
                    list.addAll(setDetail(11,value,subAgencyId,date));
                }
            }

            //合法:通过了四次判断，证明可建立该排课


        }


        String result = "";
        if (list.size()>0){
            int line1 = bSlScheduleRepository.insertList(list);
            if (line1 < 1 ) {
                throw new ServiceException(ResultServiceEnums.EXCEL_IMPORT_DB_INSERT_ERROR);
            }else{
                result+="成功导入"+list.size()+"条数据。";
            }
        }

        if(lessonCodes.size()>0){
            result+="以下课程编号不存在："+ JSON.toJSONString(lessonCodes)+"。";
        }
        if(teacherCodes.size()>0){
            result+="以下教师编号不存在："+ JSON.toJSONString(teacherCodes)+"。";
        }
        if(classCodes.size()>0){
            result+="以下班级编号不存在："+ JSON.toJSONString(classCodes)+"。";
        }
        if(roomCodes.size()>0){
            result+="以下教室编号不存在："+ JSON.toJSONString(roomCodes)+"。";
        }

        return new PersistModel(1, result);
    }

    @Override
    public ScheduleResult queryMoreBSlSchedules(String week,String subAgencyId, String termCode) {
        ScheduleResult scheduleResult = new ScheduleResult();
        BTerm bTerm = bTermRepository.queryBtermByCode(termCode).get(0);
        scheduleResult.setWeekNum(bTerm.getWeek());
        Map<String,Date> map = FindTime.findTime(bTerm.getStartDate(),bTerm.getEndDate());
        Date min = new Date();
        if(map.get(week+",1") != null && !map.get(week+",1").toString().equals("")){
            min = map.get(week+",1");
        }else if (map.get(week+",2") != null && !map.get(week+",2").toString().equals("")){
            min = map.get(week+",2");
        }else if (map.get(week+",3") != null && !map.get(week+",3").toString().equals("")){
            min = map.get(week+",3");
        }else if (map.get(week+",4") != null && !map.get(week+",4").toString().equals("")){
            min = map.get(week+",4");
        }else if (map.get(week+",5") != null && !map.get(week+",5").toString().equals("")){
            min = map.get(week+",5");
        }else if (map.get(week+",6") != null && !map.get(week+",6").toString().equals("")){
            min = map.get(week+",6");
        }else if (map.get(week+",7") != null && !map.get(week+",7").toString().equals("")){
            min = map.get(week+",7");
        }
        Calendar minCal = Calendar.getInstance();
        minCal.setTime(min);
        Calendar maxCal = Calendar.getInstance();
        maxCal.setTime(min);
        while(maxCal.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY){
            maxCal.add(Calendar.DATE,1);
        }
        SimpleDateFormat format0 = new SimpleDateFormat("yyyy-MM-dd");
        scheduleResult.setBSlSchedules(bSlScheduleRepository.queryMoreBSlSchedules(
                format0.format(minCal.getTime()),
                format0.format(maxCal.getTime()),
                subAgencyId));
        BCourseCode bCourseCode = new BCourseCode();
        bCourseCode.setFlag("1");
        bCourseCode.setDelFlag("0");
        scheduleResult.setBCourseCodes(bCourseCodeRepository.queryBCourseCodes(bCourseCode));
        return scheduleResult;
    }

    public List<BSlSchedule> setDetail(int i,String[] value,String subAgencyId,Date date){
        List<BSlSchedule> list = new ArrayList<>();
        int location1 = value[i].indexOf("-");
        String start1 = value[i].substring(0,location1);
        int numStart1 = Integer.valueOf(start1);
        String end1 = value[i].substring(location1 + 1,value[i].length());
        int numEnd1 = Integer.valueOf(end1);
        //按照星期几划分,将所有当前天的课程放入一个List集合
        for(int j = numStart1 ; j <= numEnd1 ; j++){
            BSlSchedule bSlSchedule = new BSlSchedule();
            bSlSchedule.setTeaCode(value[1]);
            bSlSchedule.setSeq(String.valueOf(j));
            bSlSchedule.setRoomCode(value[3]);
            bSlSchedule.setSubAgencyId(subAgencyId);
            bSlSchedule.setStatus("0");
            bSlSchedule.setSlState("0");
            bSlSchedule.setStartDay(date);
            bSlSchedule.setLessonCode(value[0]);
            bSlSchedule.setEduClassCode(value[2]);
            bSlSchedule.setTermCode(termCode);
            UserUtil.setCurrentPersistOperation(bSlSchedule);
            list.add(bSlSchedule);
        }

        return list;
    }

}
