package org.rcisoft.sys.user.dao;


import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bagency.entity.BAgency;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.springframework.security.access.method.P;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;

import static javafx.scene.input.KeyCode.V;

/**
 * Created by lcy on 17/8/9.
 */
@Repository
public interface SysUserMapper extends BaseMapper<SysUser> {


    @Select("<script>select * from s_user where login_name = #{username} and del_flag = 0 and flag = 1</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryUserByName(String username);

    @Insert("<script>INSERT INTO s_user" +
            "(create_date,update_date,create_by,update_by,remarks,del_flag,flag,business_id,login_name,password,name,sex,git_lab_id,git_lab_username,git_lab_impression_token,head_pic,email,agency_code)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.createBy},#{item.updateBy},#{item.remarks},#{item.delFlag},#{item.flag},#{item.businessId},#{item.loginName},#{item.password},#{item.name},#{item.sex},#{item.gitLabId},#{item.gitLabUsername},#{item.gitLabImpressionToken},#{item.headPic},#{item.email},#{item.agencyCode})" +
            "</foreach></script>")
    int insertList(List<SysUser> users);

    @Insert("<script>INSERT INTO s_r_user_role" +
            "(business_id,user_id,role_id)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.userId},#{item.roleId})" +
            "</foreach></script>")
    int insertUserRoleList(List<UserRole> users);

    @Insert("INSERT INTO s_r_user_role"+
            "(business_id,user_id,role_id)VALUES" +
            "(#{businessId},#{userId},#{roleId})")
    int insertUserRole(UserRole userRole);

    @Select("SELECT u.business_id from s_user u left join ${tab} s on u.login_name = s.code where s.business_id = #{id}")
    String selectUserIdByStuOrTeaId(@Param("id") String id, @Param("tab") String table);

    @Update("update s_user set del_flag = 1, flag = 0 where login_name = #{code}")
    int deleteByCode(String code);

    @Select("<script>select * from s_user where business_id = #{userId} and password = #{password} limit 1</script>")
    @ResultMap(value = "BaseResultMap")
    SysUser checkPassword(@Param("userId") String userId, @Param("password") String password);

    @Update("update s_user set password =#{password} where business_id = '${userId}'")
    int changePassword(@Param("userId") String userId, @Param("password") String password);

    @Select("<script>select password from s_user where business_id = #{userId} </script>")
    String queryPasswordById(String userId);

    @Update("<script>update s_user set password = #{password} where 1=1 " +
            "<if test=\"userCode !=null and userCode != '' \">and login_name = #{userCode} </if>" +
            "<if test=\"businessId !=null and businessId != '' \">and business_id = #{businessId} </if> " +
            "and del_flag = 0 and flag = 1</script>")
    int resetPassword(@Param("userCode") String userCode, @Param("businessId") String businessId, @Param("password") String password);

    @Select("<script>select business_id from s_user where login_name = #{studentCode} and del_flag = 0 and flag = 1</script>")
    String queryUserIdByLoginName(String studentCode);

    @Delete("<script>delete from s_r_user_role where user_id = #{businessId}</script>")
    int deleteUserRole(String businessId);

    @Select("<script>select su.*,ba.`name` as agencyName,sr.role_name as roleName,sr.business_id as roleId from s_user su " +
            "LEFT JOIN b_agency ba on su.agency_code = ba.`code` " +
            "LEFT JOIN s_r_user_role sru on su.business_id = sru.user_id " +
            "LEFT JOIN s_role sr on sru.role_id = sr.business_id " +
            "where su.is_created = '1' and su.del_flag = 0 and su.flag = 1  " +
            "<if test=\"agencyCode !=null and agencyCode != '' \">and su.agency_code = #{agencyCode} </if></script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryCreatedUsers(SysUser sysUser);

    @Select("<script>select agency_code from s_user where business_id = #{businessId}</script>")
    String queryAgencyCodeByBusinessId(String businessId);

    @Select("select * from b_agency where code = #{agencyCode} and del_flag = 0 and flag = 1")
    List<BAgency> checkTeacherAgency(SysUser sysUser);

    @Update("update s_user set flag =#{flag} where login_name = #{loginName} and del_flag = 0 and flag = 1")
    int stopOrStartUser(SysUser user);
}
